use tokio_cron_scheduler::{Job, JobScheduler, JobSchedulerError};

pub struct Task {
    pub name: String,
    pub job: Box<dyn Fn() + Send + Sync + 'static>,
    pub trigger: String
}

/// 启动定时任务
pub async fn start(tasks: Vec<Task>) {
    match schedule(tasks).await {
        Ok(_) => tracing::info!("定时任务启动成功"),
        Err(err) => tracing::error!(error = ?err, "定时任务启动失败"),
    }
}

async fn schedule(tasks: Vec<Task>) -> Result<(), JobSchedulerError> {
    let mut scheduler = JobScheduler::new().await?;
    scheduler.init().await?;

    // 添加任务
    for task in tasks {
        let task: &'static Task = Box::leak(Box::new(task));
        let _task_uuid = scheduler.add(Job::new_async(task.trigger.as_str(), move |_uuid, _l| {
            Box::pin(async move {
                (&task.job)();
            })
        })?).await?;
    }

    // 添加关闭监听
    scheduler.set_shutdown_handler(Box::new(|| {
        Box::pin(async move {
            tracing::info!("定时任务已关闭");
        })
    }));

    // 启动任务
    scheduler.start().await?;
    Ok(())
}
