extern crate proc_macro2;
extern crate quote;
extern crate syn;

extern crate proc_macro;

use parse::{Parse, ParseStream};
use proc_macro::{Span, TokenStream};
use punctuated::Punctuated;
// 用于处理宏输入
use quote::quote;
use spanned::Spanned;
// 用于生成代码
use syn::*;
use syn::{parse_macro_input, ItemFn};

struct RouteArgs {
    path: String,
    methods: Vec<String>,
}

// 实现 Parse trait 以支持解析参数
impl Parse for RouteArgs {
    fn parse(input: ParseStream) -> syn::parse::Result<Self> {
        // 使用Meta解析 begin
        // let args = Punctuated::<syn::Meta, syn::Token![,]>::parse_terminated(input)?;
        // let mut path = None;
        // let mut methods = Vec::new();
        // for arg in args {
        //     if let Meta::NameValue(nv) = arg {
        //         if nv.path.is_ident("method") {
        //             if let Expr::Array(ExprArray { elems, .. }) = nv.value {
        //                 for elem in elems {
        //                     if let Expr::Lit(syn::ExprLit { lit: Lit::Str(lit_str), .. }) = elem {
        //                         methods.push(lit_str.value());
        //                     }
        //                 }
        //             }
        //         } else if nv.path.is_ident("path") {
        //             if let Expr::Lit(ExprLit { lit: Lit::Str(lit_str), .. }) = nv.value {
        //                 path = Some(lit_str.value());
        //             }
        //         }
        //     }
        // }
        // let path = path.expect("Expected a path argument");
        // 使用Meta解析 end

        let args = Punctuated::<syn::Expr, syn::Token![,]>::parse_terminated(input)?;

        let mut path = None;
        let mut methods = Vec::new();

        for expr in args {
            match expr {
                Expr::Lit(ExprLit {
                    lit: Lit::Str(lit_str),
                    ..
                }) => {
                    path = Some(lit_str.value());
                }
                Expr::Assign(assign) => {
                    if let Expr::Path(path) = *assign.left {
                        if path.path.is_ident("method") {
                            if let Expr::Array(ExprArray { elems, .. }) = *assign.right {
                                for elem in elems {
                                    if let Expr::Lit(ExprLit {
                                        lit: Lit::Str(lit_str),
                                        ..
                                    }) = elem
                                    {
                                        methods.push(lit_str.value());
                                    }
                                }
                            }
                        }
                    }
                }
                _ => {}
            }
        }

        let path = path.expect("路由参数不能为空");

        Ok(RouteArgs { path, methods })
    }
}

struct Args {
    vars: Vec<syn::Expr>,
}

impl Parse for Args {
    fn parse(input: ParseStream) -> syn::parse::Result<Self> {
        let vars = Punctuated::<syn::Expr, syn::Token![,]>::parse_terminated(input)?;

        Ok(Args {
            vars: vars.into_iter().collect(),
        })
    }
}

impl Args {
    pub fn get_arg(&self, index: usize) -> syn::Result<Option<syn::Expr>> {
        match self.vars.get(index) {
            Some(var) => Ok(Some(var.clone())),
            None => {
                // 第一个参数使路由url，必须存在，其他的参数根据实际需求进一步解析
                if index != 0 {
                    return Ok(None);
                } else {
                    return Err(syn::Error::new(
                        Span::call_site().into(),
                        "route must have one argument",
                    ));
                }
            }
        }
    }
}

pub fn gen_route(attr: TokenStream, item: TokenStream, method: &str) -> TokenStream {
    let func = parse_macro_input!(item as ItemFn);
    // 解析宏的参数
    let RouteArgs { path, methods } = parse_macro_input!(attr as RouteArgs);

    // 将 method 转换为小写并生成标识符
    let vis = func.vis.clone();
    let ident = func.sig.ident.clone();

    let mut method_routers = Vec::new();
    if methods.is_empty() {
        let method_name = syn::Ident::new(&method.to_lowercase(), method.span());
        method_routers.push(quote! {
            (#path, axum::routing::#method_name(#ident))
        });
    } else {
        for method in methods {
            let method_name = syn::Ident::new(&method.to_lowercase(), method.span());
            method_routers.push(quote! {
                (#path, axum::routing::#method_name(#ident))
            });
        }
    }

    let generated = quote! {
        #vis fn #ident () -> Vec<(&'static str, axum::routing::method_routing::MethodRouter)> {
            #func

            vec![#(#method_routers),*]
        }
    };

    TokenStream::from(generated)
}

#[allow(dead_code)]
pub fn gen_dyn_route(attr: TokenStream, item: TokenStream, method: &str) -> TokenStream {
    let args = parse_macro_input!(attr as Args);
    let func = parse_macro_input!(item as ItemFn);

    let vis = func.vis.clone();
    let ident = func.sig.ident.clone();

    let route = args.get_arg(0).unwrap().unwrap();

    let method_option = args.get_arg(1);
    match method_option {
        Ok(method) => {
            if let Some(Expr::Assign(methods_value)) = method {
                println!("method is {:?}", methods_value);
                // if let Expr::Path(left) = methods_value.left {
                //     left.path.segments[0].ident.eq("d") {

                //     }
                // }
                // if let Expr::Lit(right) = methods_value.right {

                // }
            }
        }
        Err(err) => {
            println!("error is {}", err);
        }
    }

    let method_name: Ident = Ident::new(method, route.span());

    let expanded = quote! {
        #vis fn #ident () -> (&'static str, axum::routing::method_routing::MethodRouter) {
            #func

            (#route, axum::routing::#method_name(#ident))
        }
    };
    expanded.into()
}
