use domain::dto::social_wx::WxLoginInfo;
use i18n::{message, message_ids::MessageId};
use library::{
    context::Context,
    model::response::{ResErr, ResResult},
    social::wechat::{MiniAppLoginResult, WeChatAccessToken, WECHAT_SOCIAL},
};

/// 获取微信access_token
pub async fn get_wechat_access_token(context: Context) -> ResResult<WeChatAccessToken> {
    let lang_tag = context.get_lang_tag();
    let access_token = match WECHAT_SOCIAL.get_access_token().await {
        Ok(access_token) => access_token,
        Err(err) => {
            tracing::error!("获取微信access_token失败,err:{}", err);
            return Err(ResErr::service(message!(
                lang_tag,
                MessageId::FailedGetWxAaccessToken
            )));
        }
    };
    Ok(access_token)
}

/// 微信登录，code通过wx.login获取
/// 
/// todo 登录成功后，判断用户是否存在，不存在，则注册用户，存在，则更新用户信息
pub async fn code_2_session(context: Context, code: String) -> ResResult<MiniAppLoginResult> {
    let lang_tag = context.get_lang_tag();
    let result = match WECHAT_SOCIAL.code_2_session(&code).await {
        Ok(result) => result,
        Err(err) => {
            tracing::error!("微信登录失败,err:{}", err);
            return Err(ResErr::service(message!(
                lang_tag,
                MessageId::FailedWeChatLogin
            )));
        }
    };
    Ok(result)
}

/// 微信登录校验
/// 
/// todo 校验成功后，判断用户是否存在
pub async fn check_session(context: Context, wx_login_info: WxLoginInfo) -> ResResult<()> {
    let lang_tag = context.get_lang_tag();
    let session_key = wx_login_info.session_key.unwrap();
    let open_id = wx_login_info.openid.unwrap();
    match WECHAT_SOCIAL.check_session(&session_key, &open_id).await {
        Ok(_) => Ok(()),
        Err(err) => {
            tracing::error!("微信登录校验失败,err:{}", err);
            Err(ResErr::service(message!(
                lang_tag,
                MessageId::FailedWeChatLogin
            )))
        }
    }
}